/*
 * @(#)55    1.1  src/pdwebrte/azn/eaidemo/EAIDemo/JavaSource/com/tivoli/am/eb/web/eaidemo/EAIDemoServerConfig.java, pdweb.ws.authn.eai, amwebrte600, 040916a  8/20/04  16:50:08
 * ===========================================================================
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2004 All Rights Reserved.
 * ===========================================================================
 *
 */
package com.tivoli.am.eb.web.eaidemo;


import java.io.FileInputStream;
import java.io.FileOutputStream;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.Properties;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;


/**
 * Class representing EAI servlet configuration parameters. The parameters are
 * stored as servlet context parameters and can be changed by editing the
 * <code>web.xml</code> file for this application.
 *
 * @author Tivoli Systems
 */
public class EAIDemoServerConfig {
    /**
     * Indicates the simple demo, returning only User ID headers.
     */
    public static final short DEMO_TYPE_USER_ID = 0;


    /**
     * Indicates the advanced demo, returning only PAC headers and requiring
     * configuration through PdJrteCfg and SvrSslCfg.
     */
    public static final short DEMO_TYPE_PAC = 1;
    private static EAIDemoServerConfig instance = null;
    private Properties config = null;
    private ServletContext context = null;


    /**
     * Creates a new EAIDemoServerConfig object.
     *
     * @param ctx The ServletContext where EAI parameters may be found.
     *
     * @throws ServletException If either the eai-user-id-header parameter or
     *         the eai-pac-header parameters are missing from the context
     *         parameters.
     */
    private EAIDemoServerConfig(ServletContext ctx) throws ServletException {
        context = ctx;
        config = new Properties();

        try {
            config.load(new FileInputStream(context.getInitParameter("eai-props-file-location")));
        } catch(Exception ex) {
            System.err.println(ex.getMessage());
            createEmptyConfig();
        }
    }


    /**
     * Singleton pattern to return the common configuration object.
     *
     * @param ctx the ServletContext for this application.
     *
     * @return The config singleton, or a new config object if the singleton is
     *         null.
     *
     * @throws ServletException if there's a problem with the config.
     */
    public static EAIDemoServerConfig getInstance(ServletContext ctx)
                                           throws ServletException
    {
        if(instance == null) {
            instance = new EAIDemoServerConfig(ctx);
        }

        return instance;
    }


    /**
     * Returns the properties object representing the config file.
     *
     * @return the properties object.
     */
    public Properties getConfig() {
        return config;
    }


    /**
     * Returns the value corresponding to the eai-auth-level header.
     *
     * @param value the default auth level if the user doesn't supply one.
     */
    public void setDefaultAuthLevelValue(String value) {
        config.setProperty("auth-level-value", value);
    }


    /**
     * Returns the value corresponding to the eai-auth-level header.
     *
     * @return The auth level, a string between 1 and 5; null if the parameter
     *         does not exist.
     */
    public String getDefaultAuthLevelValue() {
        return config.getProperty("auth-level-value");
    }


    /**
     * Returns the value of the URL to which WebSEAL should redirect users upon
     * successful authentication.
     *
     * @param value the default redirect url to set.
     */
    public void setDefaultRedirectURL(String value) {
        config.setProperty("redirect-url-value", value);
    }


    /**
     * Returns the value of the URL to which WebSEAL should redirect users upon
     * successful authentication.
     *
     * @return A String representation of the URL for inclusion in the reponse
     *         headers.
     */
    public String getDefaultRedirectURL() {
        return config.getProperty("redirect-url-value");
    }


    /**
     * The TAM user that all EAI demo logins map to. This user <b>must</b>
     * exist in the TAM registry.
     *
     * @param value the username that will be used for credential generation.
     */
    public void setDefaultUser(String value) {
        config.getProperty("tam-eai-user", value);
    }


    /**
     * The TAM user that all EAI demo logins map to. This user <b>must</b>
     * exist in the TAM registry.
     *
     * @return The user ID; null if the parameter does not exist.
     */
    public String getDefaultUser() {
        return config.getProperty("tam-eai-user");
    }


    /**
     * Returns the name of the header that should be used to indicate
     * authentication level.
     *
     * @param value the name of the auth level header.
     */
    public void setEAIAuthLevelHeader(String value) {
        config.setProperty("eai-auth-level-header", value);
    }


    /**
     * Returns the name of the header that should be used to indicate
     * authentication level.
     *
     * @return The header name; null if the parameter does not exist.
     */
    public String getEAIAuthLevelHeader() {
        return config.getProperty("eai-auth-level-header");
    }


    /**
     * Returns the name of the header that should be used to indicate the user
     * PAC.
     *
     * @param value the name of the PAC header.
     */
    public void setEAIPACHeader(String value) {
        config.setProperty("eai-pac-header", value);
    }


    /**
     * Returns the name of the header that should be used to indicate the user
     * PAC.
     *
     * @return The header name; null if the parameter does not exist.
     */
    public String getEAIPACHeader() {
        return config.getProperty("eai-pac-header");
    }


    /**
     * Returns the name of the header that should be used to indicate the PAC
     * service control header.
     *
     * @param value the name of the PAC service control header.
     */
    public void setEAIPACSvcHeader(String value) {
        config.setProperty("eai-pac-svc-header", value);
    }


    /**
     * Returns the name of the header that should be used to indicate the PAC
     * service control header.
     *
     * @return The header name; null if the parameter does not exist.
     */
    public String getEAIPACSvcHeader() {
        return config.getProperty("eai-pac-svc-header");
    }


    /**
     * Returns the name of the header that should be used to indicate the URL
     * to which WebSEAL will redirect users upon successful authentication.
     *
     * @param value the name of the redirection url header.
     */
    public void setEAIRedirURLHeader(String value) {
        config.setProperty("eai-redir-url-header", value);
    }


    /**
     * Returns the name of the header that should be used to indicate the URL
     * to which WebSEAL will redirect users upon successful authentication.
     *
     * @return The header name; null if the parameter does not exist.
     */
    public String getEAIRedirURLHeader() {
        return config.getProperty("eai-redir-url-header");
    }


    /**
     * Returns the name of the header that should be used to indicate the user
     * ID for WebSEAL to use when constructing a credential for the user.
     *
     * @param value the name of the user id header.
     */
    public void setEAIUserIDHeader(String value) {
        config.setProperty("eai-user-id-header", value);
    }


    /**
     * Returns the name of the header that should be used to indicate the user
     * ID for WebSEAL to use when constructing a credential for the user.
     *
     * @return The header name; null if the parameter does not exist.
     */
    public String getEAIUserIDHeader() {
        return config.getProperty("eai-user-id-header");
    }


    /**
     * Returns the name of the header that should be used to indicate EAI
     * extended attributes.
     *
     * @param value the name of the extended attributes header.
     */
    public void setEAIxattrsHeader(String value) {
        config.setProperty("eai-xattrs-header", value);
    }


    /**
     * Returns the name of the header that should be used to indicate EAI
     * extended attributes.
     *
     * @return The header name; null if the parameter does not exist.
     */
    public String getEAIxattrsHeader() {
        return config.getProperty("eai-xattrs-header");
    }


    /**
     * Returns the URL of the PDJRTE configuration file.
     *
     * @param value the url to the PDJRTE configuration file.
     */
    public void setPDPermPropertiesURL(String value) {
        config.setProperty("jrte-cfg-url", value);
    }


    /**
     * Returns the URL of the PDJRTE configuration file.
     *
     * @return The location of the configuration file in URL format.
     *
     * @throws MalformedURLException If the configuration file contains an
     *         incorrectly formatted URL.
     */
    public URL getPDPermPropertiesURL() throws MalformedURLException {
        return new URL(config.getProperty("jrte-cfg-url"));
    }


    /**
     * Saves the config file to disk.
     */
    public void storeConfig() {
        try {
            config.store(new FileOutputStream(context.getInitParameter("eai-props-file-location")),
                         null);
        } catch(Exception ex) {
            ex.printStackTrace();
        }
    }


    /**
     * Generates a default config file if none exists.
     */
    private void createEmptyConfig() {
        config.setProperty("eai-pac-header", "am-eai-pac");
        config.setProperty("eai-pac-svc-header", "am-eai-pac-svc");
        config.setProperty("eai-user-id-header", "am-eai-user-id");
        config.setProperty("eai-auth-level-header", "am-eai-auth-level");
        config.setProperty("eai-xattrs-header", "am-eai-xattrs");
        config.setProperty("eai-redir-url-header", "am-eai-redir-url");
        config.setProperty("auth-level-value", "0");
        config.setProperty("redirect-url-value", "/");
        config.setProperty("tam-eai-user", "testuser");
        config.setProperty("jrte-cfg-url", "");
    }
}
